/* 
FILENAME : LIDAR_RADAR.ino
AUTHOR : FHN
DATE : 17-MAY-2019
MCU : ARDUINO-NANO
SUPPORT : A4988 + STEPPER + VL53L0X(LIDAR)
*/

// set variables
unsigned char PIXEL[8]={0x01,0x02,0x04,0x08,0x10,0x20,0x40,0x80};
const int STEPPER_DELAY = 100;  // delay to settle the stepper
const int stepsPerRevolution = 200;  // set the number of steps per revolution for your motor
int ONESTEP=2;   // number of step signals sent to A4988 per loop
int CW = 1;
int CCW = -1;

const int LED_LEFT = 0;
const int LED_OBST = A3;
const int LED_RIGHT = 1;
const int MOTOR_DIR = A0;
const int MOTOR_STEP = A1;
const int GLCD_DATA0 = 6;
const int GLCD_DATA1 = 7;
const int GLCD_EN = 5;
const int GLCD_RS = 4;
const int GLCD_CS2 = 3;
const int GLCD_CS1 = 2;

#include "GLCDfont.H"
#include <Wire.h>  
#include "VL53L0X.h"  

VL53L0X Lidar1;
char TEMPSTR[15];
long DISTANCE=0;

void setCS ( int cs1, int cs2 )
{
  digitalWrite(GLCD_CS1, cs1);
  digitalWrite(GLCD_CS2, cs2);
}

void GLCD_write ( unsigned char ch)
{
  PORTB = (ch>>2);   
  digitalWrite(GLCD_DATA0, ch&0x01);
  digitalWrite(GLCD_DATA1, ((ch&0x02)>>1));
  digitalWrite(GLCD_EN, HIGH);
  _delay_us(5);
  digitalWrite(GLCD_EN, LOW);
  _delay_us(5);
}

void GLCD_writeCommand(unsigned char com)   
{
  digitalWrite(GLCD_RS, LOW);
  GLCD_write ( com );
}

void GLCD_writeData(unsigned char dat)      
{
  digitalWrite(GLCD_RS, HIGH);
  GLCD_write ( dat );
}


void GLCD_writeChar(char ch)      
{
  digitalWrite(GLCD_RS, HIGH);
  for ( int col=0; col<GCLD_FONT_WIDTH; col++)
      GLCD_writeData (glcdfont[ch-32][col]);
  GLCD_writeData(0);
}

void GLCD_writeString ( char str[] , int len)
{
  for ( int i=0; str[i]!=0 && i<len; i++)
  {
    GLCD_writeChar ( str[i] );
  }
}

void showDistValue ( long dist )
{
  int V = dist/10;
  GLCD_gotoRowCol ( 7, 17 );
  GLCD_writeString ( "cm" , 2);
  GLCD_gotoRowCol ( 7, 15 );
  if (V>99)
  {
    GLCD_writeString ( "OUT  " , 5);
    return;
  }
  
  V=V%100;
  TEMPSTR[0]=V/10 + '0';
  TEMPSTR[1]=V%10 + '0';
  TEMPSTR[2]=0;

  GLCD_writeString ( TEMPSTR , 2);
}


void GLCD_Init()         
{
  setCS ( HIGH, HIGH );

  _delay_ms(20);
  GLCD_writeCommand(0x3E); 
  GLCD_writeCommand(0x40); 
  GLCD_writeCommand(0xB8); 
  GLCD_writeCommand(0xC0); 
  GLCD_writeCommand(0x3F); 
}

void GLCD_clrscr()        
{
  int i,j;
   setCS (HIGH,HIGH);

  for(i = 0; i < 8; i++)
  {
    GLCD_writeCommand((0xB8) + i); 
    for(j = 0; j < 64; j++)
    {
      GLCD_writeData(0);      
    }
  }
  GLCD_writeCommand(0x40);        
  GLCD_writeCommand(0xB8);        
  GLCD_writeCommand(0xC0);        
}

void GLCD_gotoRowCol ( int row, int col )
{
  if (col<10)
    setCS ( HIGH,LOW);
  else if (col<20)
    setCS ( LOW, HIGH );
  else
    return;  // out of screen
    
  GLCD_writeCommand(0x40+((col%10)*6));   
  GLCD_writeCommand(0xB8+row);        
}

void GLCD_writePixel ( int x, int y ) 
{
  if (x<64)
  {
    setCS ( HIGH,LOW);
  }
  else if (x<128)
  {
    setCS ( LOW,HIGH);
  }
  else
    return;  // out of screen

  int row = y/8;
  GLCD_writeCommand(0x40+(x%64));  
  GLCD_writeCommand(0xB8+row);     
  if (row==6)
    GLCD_writeData ( 0x80 | PIXEL[y%8]);
  else
    GLCD_writeData ( PIXEL[y%8]);
}


void GLCD_showLineFinal ( int dist, float xstep )
{
    int cx=64;
    int x=64;
    int prev_x=64;
    int ch=0;
    for ( int y=0; y<dist && y<64; y++ )
    {
      x=cx+y*xstep;
      GLCD_writePixel (x,63-y);
    }
}


void rotateMotor ( int dir, int steps )
{
  if (dir>0)
    digitalWrite(MOTOR_DIR, HIGH);  
  else
    digitalWrite(MOTOR_DIR, LOW);  
    
  for ( int i=0; i<steps; i++)
  {
    digitalWrite(MOTOR_STEP, HIGH);  
    delay ( 2 );
    digitalWrite(MOTOR_STEP, LOW);  
    delay ( 2 );
  }
  
}

void checkObstacle ( long dist , int p )
{
    digitalWrite(LED_OBST, HIGH);  
    if (dist<1000)
    {
      digitalWrite(LED_OBST, HIGH);  
      if ( p<0 )
        digitalWrite(LED_LEFT, HIGH);  
      else
        digitalWrite(LED_RIGHT, HIGH);  
    }
    else
      digitalWrite(LED_OBST, LOW);  
}

void setup() 
{
  DDRB = B00111111;
  DDRD = B11111110;
  pinMode(LED_LEFT, OUTPUT);
  pinMode(LED_OBST, OUTPUT);
  pinMode(LED_RIGHT, OUTPUT);
  pinMode(MOTOR_DIR, OUTPUT);
  pinMode(MOTOR_STEP, OUTPUT);
  pinMode(GLCD_DATA0, OUTPUT);
  pinMode(GLCD_DATA1, OUTPUT);
  pinMode(GLCD_EN, OUTPUT);
  pinMode(GLCD_RS, OUTPUT);
  pinMode(GLCD_CS1, OUTPUT);
  pinMode(GLCD_CS2, OUTPUT);

   _delay_ms(50);
   
  GLCD_Init();
  _delay_ms(50);
  GLCD_clrscr();  
 
  GLCD_gotoRowCol ( 3, 1 );
  GLCD_writeString ("LIDAR",5 );
  GLCD_gotoRowCol ( 4, 4 );
  GLCD_writeString ("RADAR",5 );
    
  Wire.begin();
  delay(10);

  GLCD_gotoRowCol ( 5, 1 );
  GLCD_writeString ("WireOK",6 );

  _delay_ms(50);
  
  Lidar1.init();
  delay(10);
  Lidar1.setTimeout(500);
  Lidar1.setMeasurementTimingBudget(200000);

  GLCD_gotoRowCol ( 7, 1 );
  GLCD_writeString ("LidarSet",8 );
  
  delay(3000);
  
//  delay(1500);
  GLCD_clrscr();   
  delay(100);
  GLCD_gotoRowCol ( 3, 1 );
  GLCD_writeString ("ADJUSTING",9 );
  GLCD_gotoRowCol ( 3, 11 );
  GLCD_writeString ("STEPPER",7 );
  delay(500);

  // adjusting stepper to inital position    
  for ( int i=0; i<8; i++ )
  {
    rotateMotor ( CCW, ONESTEP );
    delay(STEPPER_DELAY*2);
  }
}

void loop()
{
  delay(500);
  GLCD_clrscr();   
  GLCD_gotoRowCol ( 6, 1 );
  GLCD_writeString ("LIDAR",5 );
  GLCD_gotoRowCol ( 7, 1 );
  GLCD_writeString ("RADAR",5 );
  
  digitalWrite(LED_LEFT, LOW);  
  digitalWrite(LED_OBST, LOW); 
  digitalWrite(LED_RIGHT, LOW); 
  for ( int i=8; i>=-8; i-- )
  {
    if(i!=8)
      rotateMotor ( CW, ONESTEP );
    delay(STEPPER_DELAY);
    DISTANCE = Lidar1.readRangeSingleMillimeters();
    showDistValue ( DISTANCE );
    checkObstacle(DISTANCE, i);
    GLCD_showLineFinal ( DISTANCE/16, 0.12*i );  
  }

  delay(500);
  GLCD_clrscr();  
  GLCD_gotoRowCol ( 6, 1 );
  GLCD_writeString ("LIDAR",5 );
  GLCD_gotoRowCol ( 7, 1 );
  GLCD_writeString ("RADAR",5 );

  digitalWrite(LED_LEFT, LOW);  
  digitalWrite(LED_OBST, LOW); 
  digitalWrite(LED_RIGHT, LOW); 
  for ( int i=-8; i<=8; i++ )
  {
    if(i!=-8)
      rotateMotor ( CCW, ONESTEP );
    delay(STEPPER_DELAY);
    DISTANCE = Lidar1.readRangeSingleMillimeters();
    showDistValue ( DISTANCE );
    checkObstacle(DISTANCE, i); 
    GLCD_showLineFinal ( DISTANCE/16, 0.12*i );  // 20 for 128cm and 15 for 96cm distance
  }
}

